/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : LCD_LIB.c
* Version      : Rev. 1.00
* Device(s)    : R5F100GLG
* Tool-Chain   : CCRL
* Description  : This file implements Arduino library(Wire).
* Creation Date: 2021/02/01 (RL78/G14)
* Modified Date: 2023/12/22 port to RL78/G23
********************************************************************************/


/*******************************************************************************
Global variables and functions
********************************************************************************/
#define     SLADDR_LCD  ( 0b1010000 )   // I2C bus slave address of LCD display
#define     COMBYTE     ( 0x00 )        // control byte for command
#define     DATABYTE    ( 0x80 )        // control byte for data
#define     CLRDISP     ( 0x01 )        // clear display instruction
#define     HOMEPOSI    ( 0x02 )        // home position instruction
#define     LCD_Mode    ( 0b00111000 )  // operation mode instruction
/*                          ++||||------ must be 0
                              +|||------ must be 1
                               ||+------ 5*10/5*8dots
                               |+------- 2line/1line
                               +-------- data lenght 8bit(FIX)
*/
#define     DISPON      ( 0b00001111 )  // display on instruction
/*                          ++++||||---- must be 0
                                +|||---- must be 1
                                 ||+---- blinking of cursor
                                 |+----- cursor
                                 +------ display
*/
#define     ENTRY_Mode  ( 0b00000110 )  // entry mode instruction
/*                         ++++++|||---- must be 0
                                 +||---- must be 1
                                  |+---- shift
                                  +----- shift direction
*/



uint8_t init_LCD(void);                 // LCD initialize function
/*
    00 : success
    02 : No_slave
    03 : No ACK for data
*/

void print_LCD(uint8_t *point,uint8_t *point2);
void move_cursor(uint8_t col,uint8_t row);// move cursor to specified position
uint8_t set_data(uint8_t datacode);         // set data to LCD
uint8_t set_buff(uint8_t *point, uint8_t num);// set block data

