/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2020-2022 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c or Main.cpp                                    */
/*  DATE        :                                                      */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :RL78G23                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include <Config_IICA0.h>
#include <Config_IICA1.h>
#include <Config_TAU0_3.h>
#include <Config_TAU0_4.h>
#include <Config_TAU0_5.h>
#include <Config_TAU0_6.h>
#include <Config_TAU0_7.h>
#include "LCD_LIB.h"
#include "r_cg_userdefine.h"

/************************************************************************
Global variables
************************************************************************/


// LCD display buffer aera 40characters 2lines
// display data is 16 characters/line and 2lines.
// charactor position     0123456789012345
uint8_t disp_line1[40] = " Temp. =  15.0 C";
uint8_t disp_line2[40] = " Humidity =  50%";

volatile uint8_t sel_IICA;              // select IICA channel
volatile uint8_t sensor_sel = 0x00;     // 0x00 : HDC1080, 0x01 : HS3001
volatile uint8_t result;

/*--------------------------------------
    result of HDC1080
----------------------------------------*/
extern volatile int16_t    temp1;       // Temperature data (0.1degree unit)
extern volatile uint8_t     humid1;     // Humidity data(unit %)

/*--------------------------------------
    result of HS3001
----------------------------------------*/
extern volatile int16_t    temp2;       // Temperature data (0.1degree unit)
extern volatile uint8_t     humid2;     // Humidity data(unit %)

extern volatile uint8_t    sensor_end;  // sensor complete flag

extern volatile uint8_t    s_push;      // short time push
extern volatile uint8_t    m_push;      // middle time push
extern volatile uint8_t    l_push;      // long time push

extern volatile uint8_t    second_f;    // one second flag
extern volatile uint8_t    minute_f;    // one minute flag

volatile uint16_t   old_time;           // milli seconds current time counter

/************************************************************************
Global functions
************************************************************************/
void main(void);
void R_MAIN_UserInit(void);
void R_conv_LCD(uint8_t);

/***********************************************************************

    main function

************************************************************************/

void main(void)
{

    R_MAIN_UserInit();                  // initialize 
/*---------------------------------------------------------------------
    get new  sensor data
-----------------------------------------------------------------------*/

    result = R_sensor_start(sensor_sel);// trigger sensor

    NOP();
    
    if ( SUCCESS == result )            // sensor is exist
    {
        while ( 0 == sensor_end )       // wait for sensor data
        {
            NOP();
        }
        sensor_end = 0x00;              // clear sensor ready flag

    }

/*---------------------------------------------------------------------
    get new  sensor data
-----------------------------------------------------------------------*/

    while (1)
    {
        
        R_conv_LCD(sensor_sel);         // display data
        
/*--------------------------------------
   wait for next sense timing
----------------------------------------*/


        while ( 0 == minute_f )         // wait for 1 minute
        {

            if ( 0 != l_push )          // long push of user switch
            {
                sensor_sel ^=0x01;      // change sensor
            }
            l_push = 0;

            if ( 0 != s_push )          // short push
            {
                s_push = 0;             // clear short push flag
                break;
            }

            NOP();

        }
        
        minute_f = 0x00;                // clear 1 minute flag

/*---------------------------------------------------------------------
    get new  sensor data
-----------------------------------------------------------------------*/

        result = R_sensor_start(sensor_sel);// trigger sensor

        if ( 0x00 == result )               // sensor is exist
        {
            while ( 0 == sensor_end )       // wait for sensor data
            {
                NOP();
            }
            sensor_end = 0x00;              // clear sensor ready flag

        }


    }   /* end of while(1) loop */

}


/************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before start main function.
* Arguments    : None
* Return Value : None
*************************************************************************/
void R_MAIN_UserInit(void)
{

    sel_IICA = 1;               // select IICA1 channel

    R_Config_TAU0_4_Start();    // start TM04 as switch check timer
    R_Config_TAU0_6_Start();    // start TM06 as mills seconds timer

    EI();

    do{
        old_time = ( int )( millis() & 0x0FFFF );
    }while ( old_time < 200 );  // mait for 200ms to power up time

    init_LCD();                 // initialize LCD to 2 lines mode

    disp_line1[14] = 0xDF;      // set degreeC character of LCD

    print_LCD( 
        (uint8_t *__near)disp_line1,    // display 1st line
        (uint8_t *__near)disp_line2);   // display 2nd line

}

/************************************************************************
* Function Name: R_conv_LCD
* Description  : This function make LCD display data for HDC1080
* Arguments    : sensor 0x00 : HDC1080, 0x01 : HS3001
* Return Value : None
* Remarks      : temp1,humid1
*************************************************************************/
void R_conv_LCD(uint8_t sensor_sel)
{

    int16_t     work_int;               // for work area
    int16_t     temp;                   // temperature data work(0.1degree)
    uint16_t    humid;                  // Humidity data work

/*--------------------------------------
   get temp and humid data
----------------------------------------*/
    switch ( sensor_sel )
    {
        case 0x00:                      // HDC1080
            temp    = temp1;            // get HDC1080 temperature data
            humid   = humid1;           // get HDC1080 Humidity data
            break;

        case 0x01:                      // HS3001
            temp    = temp2;            // get HS3001 temperature data
            humid   = humid2;           // get HS3001 Humidity data
            break;

        default:                        // for future expansion
            break;

    }

/*--------------------------------------
   Calculating Temperature
----------------------------------------*/

    if (temp < 0 )                      // below zero
    {
        disp_line1[9] ='-';             // set '-'
        work_int = ( 0 - temp );        // get absolute value
    }
    else
    {
        work_int = temp;
        disp_line1[9] =' ';             // display space
        if ( 1000 <= work_int )         // when temperature is higher than 100 degreeC
        {
            disp_line1[9] ='1';         // display 1
            work_int -= 1000;           // get digit or below 100degree
        }

    }

    disp_line1[13] = (( work_int % 10 ) + 0x30); // set 0.1 degreeC digit
    work_int /= 10;                     // get 1degreeC data
    disp_line1[11] = (( work_int % 10 ) + 0x30);// set 1 degreeC digit
    work_int /=10;                      // 10degreeC data
    disp_line1[10] = (( work_int % 10 ) + 0x30);// set 10 degreeC digit

/*--------------------------------------
   Calculating Humidity
----------------------------------------*/

    if ( 100 <= humid )
    {
        disp_line2[12] = '1';   // set digit 100% to 1
        disp_line2[13] = '0';   // set digit 10% to 0
        disp_line2[14] = '0';   // set digit 1% to 0
    }
    else
    {
        disp_line2[12] = ' ';   // set digit 100% to space
        disp_line2[14] = (( humid % 10 ) + 0x30);   // set 1% digit
        disp_line2[13] = (( humid / 10 ) + 0x30);   // set 10% digit
    }

/*--------------------------------------
   display data
----------------------------------------*/
    print_LCD((uint8_t *__near)disp_line1,(uint8_t *__near)disp_line2); // transmit new data to LCD display

}
