/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_IICA0.c
* Component Version: 1.5.1
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_IICA0.
***********************************************************************************************************************/
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_IICA0.h"
/* Start user code for include. Do not edit comment generated here */
#include "Config_TAU0_7.h"
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
volatile uint8_t g_iica0_master_status_flag;      /* iica0 master flag */
volatile uint8_t * gp_iica0_rx_address;           /* iica0 receive buffer address */
uint16_t g_iica0_rx_len;                          /* iica0 receive data length */
volatile uint16_t g_iica0_rx_cnt;                 /* iica0 receive data count */
volatile uint8_t * gp_iica0_tx_address;           /* iica0 send buffer address */
volatile uint16_t g_iica0_tx_cnt;                 /* iica0 send data count */

volatile uint8_t g_time_out;                    /* timeout of detect STOP/START */
volatile uint8_t g_status;                      /* iica0 master operationflag   */
/*
    0x00(BUS_FREE)  : bus is free
    0x20(OP_END)    : operation end without error
    0x20(BUS_HOLD)  : IICA0 hold I2C bus
    0x22(TX_END)    : transmit operation complete and hold I2C bus
    0x24(RX_END)    : receive operation complete and hold I2C bus
    0x12(TX_MODE)   : data transmit mode
    0x14(RX_MODE)   : data receive mode
    0x0F(BUS_BUSY)  : bus is busy
    0x8F(BUS_ERROR) : operation error (BUS or IICA0 is BUSY)
    0x80(NO_SLAVE)  : NACK for slave address
    0xC0(NO_ACK)    : NACK for data
*/

uint8_t R_IICA0_StartCondition(void);   /* if possible then issue StartCondition    */

/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_IICA0_Create
* Description  : This function initializes the IICA0 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_IICA0_Create(void)
{
    IICA0EN = 1U;    /* enables input clock supply */
    IICE0 = 0U;
    IICAMK0 = 1U;    /* disable INTIICA0 interrupt */
    IICAIF0 = 0U;    /* clear INTIICA0 interrupt flag */
    /* Set INTIICA0 level 2 priority */
    IICAPR10 = 1U;
    IICAPR00 = 0U;
    /* Set SCLA0, SDAA0 pin */
/* Start user code */
    PM6 |= 0x03U;
/* End user code */
    PMCE6 &= 0xFCU;
    CCDE &= 0xCFU;
    P6 &= 0xFCU;
    PM6 |= 0x03U;
    SMC0 = 0U;
    IICWL0 = _4C_IICA0_IICWL_VALUE;
    IICWH0 = _55_IICA0_IICWH_VALUE;
    IICCTL01 |= _01_IICA_fCLK_HALF;
    SVA0 = _10_IICA0_MASTERADDRESS;
    STCEN0 = 1U;
    IICRSV0 = 1U;
    SPIE0 = 0U;
    WTIM0 = 1U;
    ACKE0 = 1U;
    IICAMK0 = 0U;
    IICE0 = 1U;
    LREL0 = 1U;
    /* Set SCLA0, SDAA0 pin */
    PM6 &= 0xFCU;

    R_Config_IICA0_Create_UserInit();
}

/* Start user code for adding. Do not edit comment generated here */
/***********************************************************************************************************************
* Function Name: R_Config_IICA0_Master_Send
* Description  : This function starts to send data as master mode.
* Arguments    : adr -
*                    transfer address
*                tx_buf -
*                    transfer buffer pointer
*                tx_num -
*                    buffer size
*                wait -
*                    wait for start condition
* Return Value : status -
*                    MD_OK, MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
MD_STATUS R_Config_IICA0_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num, uint8_t wait)
{
    MD_STATUS status = MD_OK;

    g_time_out = wait;                  /* set timeout data to work area*/

    status = (MD_STATUS)R_IICA0_bus_check(); /* check IICA0 & I2C bus   */

    if (MD_OK == status)
    {

/*------------------------------------------------------
    success to get I2C bus (Detect start condition)
    then copy parameters for transmission
    and transmit slave address to I2C bus
------------------------------------------------------*/

        g_iica0_tx_cnt = tx_num;        /* set transmit data number     */
        gp_iica0_tx_address = tx_buf;   /* set transmit data address    */
        adr &= (uint8_t)~0x01U;         /* set send mode                */
        IICA0 = adr;                    /* send slave address           */
        g_status = TX_MODE;             /* set IICA0 operation mode     */

    }

    return (status);
}

/***********************************************************************************************************************
* Function Name: R_Config_IICA0_Master_Receive
* Description  : This function starts to receive data as master mode.
* Arguments    : adr -
*                    receive address
*                rx_buf -
*                    receive buffer pointer
*                rx_num -
*                    buffer size
*                wait -
*                    wait for start condition
* Return Value : status -
*                    MD_OK, MD_ERROR1 or MD_ERROR2
***********************************************************************************************************************/
MD_STATUS R_Config_IICA0_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num, uint8_t wait)
{
    MD_STATUS status = MD_OK;


    g_time_out = wait;                  /* set timeout data to work area    */

    status = (MD_STATUS)R_IICA0_bus_check();    /* check IICA0 & I2C bus    */

/* Detect start condition */

    if (MD_OK == status)
    {
/*------------------------------------------------------
    success to get I2C bus (Detect start condition)
    then copy parameters for receiving 
    and transmit slave address to I2C bus
------------------------------------------------------*/

        g_iica0_rx_len = rx_num;        /* set receive data number      */
        g_iica0_rx_cnt = 0U;            /* clear receive data number    */
        gp_iica0_rx_address = rx_buf;   /* set receive buffer address   */
        adr |= 0x01U;                   /* set receive mode             */
        IICA0 = adr;                    /* send slave  address          */
        g_status = RX_MODE;             /* set  IICA0 operation mode    */

    }

    return (status);
}

/*******************************************************************************
* Function Name: R_IICA0_check_comstate
* Description  : This function get communication status
* Arguments    : none
* Return Value : result status
                0x00(BUS_FREE)  : I2C bus is free(SUCCESS)
                0x14(RX_MODE)   : receive operation mode
                0x12(TX_MODE)   : transmit operation mode
                0x20(OP_END)    : operation end and hold I2C bus
                0x22(TX_END)    : transmit end and hold I2C bus
                0x24(RX_END)    : receive and and hold I2C bus
                0x8F(BUS_ERROR) : bus or IICA0 is busy error
                0x80(NO_SLAVE)  : NACK for slave address
                0xC0(NO_ACK)    : NACK for transmit data
********************************************************************************/
uint8_t R_IICA0_check_comstate(void)
{

    return( g_status );

}   /* end of R_IICA0_check_comstate*/

/*******************************************************************************
* Function Name: R_IICA0_bus_check
* Description  : This function check I2C bus and issue start condition if possible
* Arguments    : none
* Return Value : status -
*               0x00(SUCCESS)  : get I2C bus (success to get bus)
*               0x8F(BUS_ERROR): I2C bus or IICA0 is busy
********************************************************************************/
uint8_t R_IICA0_bus_check(void)
{
    uint8_t status;

    /* Check I2C bus is used or in waiting state    */

    status = BUS_ERROR;                     /* set dummy error flag         */
    if ( 0x00 == ( g_status & 0x10) )       /* check if operating or not    */
    {

/*------------------------------------------------------
    IICA0 is not COMMUNICATING and check I2C bus status
------------------------------------------------------*/

        if ( ( 1 == MSTS0 ) || ( 0 == IICBSY0 )  )  /* check if I2C is busy */
        {

/*------------------------------------------------------
    IICA0 and I2C bus are usable then
    issue StartCondition to I2C bus
------------------------------------------------------*/

            status = R_IICA0_StartCondition();  /* issue start condition    */

        }   /* end of handling when bus is available    */

    }   /* end of handling when IICA0 is free   */

    return (status);

}   /* end of R_IICA0_bus_check */


/*******************************************************************************
* Function Name: R_IICA0_StartCondition
* Description  : This function issue start condition
* Arguments    : none
* Return Value : result
                0x00(SUCCESS)   : success
                0x8F(BUS_ERROR) : error
********************************************************************************/
uint8_t R_IICA0_StartCondition(void)
{

    uint8_t status;
    uint8_t wait;

    STT0 = 1;                           /* issue IICA0 start condition  */

/*------------------------------------------------------
    Wait for detecting StartCondition  otherwise timeout
------------------------------------------------------*/

    for( wait = g_time_out ; wait > 0 ; --wait)
    {
        if ( 1 == STD0 )
        {
            break;                      /* exit with start condition    */
        }
    }

/*------------------------------------------------------
    detecte start condition or timeout
------------------------------------------------------*/

    if ( ( 1 == MSTS0 ) && ( 1 == STD0 ) )  /* check bus status         */
    {

/*------------------------------------------------------
    get bus and success to issue start condition
------------------------------------------------------*/

        status = SUCCESS;               /* set good result              */
        g_status = BUS_HOLD;            /* set new status               */
    }
    else
    {
        status = BUS_ERROR;             /* set status "error"           */
        g_status = BUS_ERROR;
    }

    return (status);

}   /* end of R_IICA0_StartCondition*/


/*******************************************************************************
* Function Name: R_IICA0_StopCondition
* Description  : This function issue stop condition
* Arguments    : none
* Return Value : result
*               0x00(SUCCESS,BUS_FREE)  : success
*               0x8F(BUS_ERROR)         : error
********************************************************************************/
uint8_t R_IICA0_StopCondition(void)
{

    uint8_t status;
    uint16_t wait;


    SPT0 = 1;                           /* issue stop condition to stop */

    for( wait = g_time_out ; wait > 0 ; wait-- )
    {                                   /* wait for detection           */
        if ( 1 == SPD0 )
        {                               /* detect stop condition        */
            status = SUCCESS;           /* set status "success"         */
            g_status = BUS_FREE;        /* set bus is free              */
            break;
        }
    }

    if ( 0 == SPD0 )
    {             
       status = BUS_ERROR;              /* not detect stop condition    */
    }

    return (status);

}   /* end of R_IICA0_StopCondition */

/*******************************************************************************
* Function Name: R_IICA0_poll
* Description  : This function check operation end or not
* Arguments    : none
* Return Value : result status
                0x00(SUCCESS)   : success(operation complete)
                0x01(ON_COMMU)  : on communication( not complete )

                0x8F(BUS_ERROR) : bus or IICA0 is busy error
                0x80(NO_SLAVE)  : NACK for slave address
                0xC0(NO_ACK)    : NACK for transmit data
********************************************************************************/
uint8_t R_IICA0_poll(void)
{
    uint8_t status;

    status = g_status;                  /* get communication status     */

    if ( 0x30 > status )                /* check status is error or not */
    {                                   /* there is no error status     */

        if ( 0x00 == (status & 0x10) )  /* check operation end or not   */
        {
            status = SUCCESS;           /* set operation success flag   */
        }
        else
        {                               /* continue communication ?     */
            status = ON_COMMU;
        }

    }   /* end without detect error */

    return (status);                    /* return with status           */

}   /* end of R_IICA0_poll  */


/*******************************************************************************
* Function Name: R_IICA0_wait_comend
* Description  : This function wait for communication end
* Arguments    : after operation issue stopcondition or not
                0x00(HOLD_BUS) : if success,not issue stopcondition
                0x01(RELEASE)  : when finish,issue stopcondition
* Return Value : result status
                0x00(SUCCESS)   : success(operation complete)

                0x8F(BUS_ERROR) : bus or IICA0 is busy error
                0x80(NO_SLAVE)  : NACK for slave address
                0xC0(NO_ACK)    : NACK for transmit data
********************************************************************************/
uint8_t R_IICA0_wait_comend(uint8_t stop_c)
{
    uint8_t status = ON_COMMU;


    while( ON_COMMU == status )
   {
        status = R_IICA0_poll();        /* check new status             */
        NOP();                          /* wait for communication end   */
    }

    if ( ( SUCCESS !=  status ) || ( 0x00 != stop_c ) )
    {   /* communication error or argument  */
        wait_50us();                    /* wait for 50us                */
        R_IICA0_StopCondition();        /* issue STOP conditiopn        */
    }

    return( status );

}   /* end of R_IICA0_wait_comend   */

/* End user code. Do not edit comment generated here */
