/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_IICA0.h
* Component Version: 1.5.1
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_IICA0.
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_iica.h"

#ifndef CFG_Config_IICA0_H
#define CFG_Config_IICA0_H

/***********************************************************************************************************************
Macro definitions (Register bit)
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

#define _10_IICA0_MASTERADDRESS                (0x10U)    /* IICA0 address */
#define _4C_IICA0_IICWL_VALUE                  (0x4CU)
#define _55_IICA0_IICWH_VALUE                  (0x55U)

/* Start user code */

/*--------------------------------------
    g_status( IICA status ) value
----------------------------------------*/

#define     BUS_BUSY    ( 0x0F )        /* I2C bus is busy              */
#define     BUS_FREE    ( 0x00 )        /* I2C bus is free              */
#define     BUS_HOLD    ( 0x20 )        /* hold I2C bus (same as above) */

//#define     TX_SADDR    ( 0x18 )        /* TX slave address             */
//#define     TX_ADDR_REG ( 0x16 )        /* TX address register          */
#define     RX_MODE     ( 0x14 )        /* receive operation mode       */
#define     TX_MODE     ( 0x12 )        /* transmit operation mode      */
#define     OP_END      ( 0x20 )        /* operation complete           */
#define     RX_END      ( 0x24 )        /* receive operation end        */
#define     TX_END      ( 0x22 )        /* transmit operation end       */

/*--------------------------------------
    return status value
----------------------------------------*/
#define     ON_COMMU    ( 0x01 )        /* on communication             */
#define     BUS_ERROR   ( 0x8F )        /* bus or IICA0 is busy error   */
#define     NO_SLAVE    ( 0x80 )        /* NACK for slave address       */
#define     NO_ACK      ( 0xC0 )        /* NACK for transmit data       */
#define     SUCCESS     ( 0x00 )        /* operation complete           */
#define     COM_ERROR   ( 0xFF )        /* other error of communication */

/*--------------------------------------
    Argument value
----------------------------------------*/
#define     HOLD_BUS    ( 0x00 )        /* does not release Bus         */
#define     RELEASE     ( 0x01 )        /* release Bus at end of communication  */
#define     DUMMY_DATA  ( 0xFF )        /* dummy data for start receive */

/* End user code */
/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
void R_Config_IICA0_Create(void);
void R_Config_IICA0_Create_UserInit(void);
/* Start user code for function. Do not edit comment generated here */
MD_STATUS R_Config_IICA0_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num, uint8_t wait);
MD_STATUS R_Config_IICA0_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num, uint8_t wait);
uint8_t R_IICA0_StopCondition(void);    /* if possible then issue StopCondition     */
uint8_t R_IICA0_bus_check(void);        /* check if Bus is enable get Bus           */
uint8_t R_IICA0_poll(void);             /* check communication status               */
uint8_t R_IICA0_wait_comend(uint8_t stop_c); /* wait for communication end          */
uint8_t R_IICA0_check_comstate(void);   /* check communication status               */

/* End user code. Do not edit comment generated here */
#endif
