/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_IICA1_user.c
* Component Version: 1.5.1
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_IICA1.
***********************************************************************************************************************/
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_IICA1.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma interrupt r_Config_IICA1_interrupt(vect=INTIICA1, bank=RB2, enable=true)
/* assign INTIICA0, use register bank2, enable multiple interrupt*/
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
extern volatile uint8_t g_iica1_master_status_flag;      /* iica1 master flag */
extern volatile uint8_t * gp_iica1_rx_address;           /* iica1 receive buffer address */
extern uint16_t g_iica1_rx_len;                          /* iica1 receive data length */
extern volatile uint16_t g_iica1_rx_cnt;                 /* iica1 receive data count */
extern volatile uint8_t * gp_iica1_tx_address;           /* iica1 send buffer address */
extern volatile uint16_t g_iica1_tx_cnt;                 /* iica1 send data count */
extern volatile uint8_t g_status1;                       /* iica1 master operationflag   */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_IICA1_Create_UserInit
* Description  : This function adds user code after initializing the IICA1.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_IICA1_Create_UserInit(void)
{
    /* Start user code for user init. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/***********************************************************************************************************************
* Function Name: r_Config_IICA1_callback_master_sendend
* Description  : This function is a callback function when IICA1 finishes master transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_Config_IICA1_callback_master_sendend(void)
{
    g_status1 = 0x20;                       /* set transfer complete flag   */
}

/***********************************************************************************************************************
* Function Name: r_Config_IICA1_callback_master_receiveend
* Description  : This function is a callback function when IICA1 finishes master reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_Config_IICA1_callback_master_receiveend(void)
{
    g_status1 = 0x24;                       /* set receive complete flag    */
}

/***********************************************************************************************************************
* Function Name: r_Config_IICA1_callback_master_error
* Description  : This function is a callback function when IICA1 master error occurs.
* Arguments    : flag -
*                    status flag
* Return Value : None
***********************************************************************************************************************/
static void r_Config_IICA1_callback_master_error(MD_STATUS flag)
{
    /* Start user code for r_Config_IICA1_callback_master_error. Do not edit comment generated here */
    uint8_t status;

    status = R_IICA1_StopCondition();   /* issue StopCondition to release I2C Bus   */

    if ( 0x00 == status )
    {                                   /* success to issue StopCondition           */
        if ( MD_NACK == flag )          /* check error code                         */
        {
            g_status1 = NO_SLAVE;       /* error is No Ack for slave address        */
        }
        else
        {
            g_status1 = NO_ACK;         /* error is No Ack for data                 */
        }
    }

    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_IICA1_master_handler
* Description  : This function is IICA1 master handler
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_Config_IICA1_master_handler(void)
{

    if ( 1 == STD1 )                        /* detect start condition           */
    {

/*----------------------------------------------
  slave address transmission end
----------------------------------------------*/

        if ( 1 == ACKD1 )
        {

/*======================================================
  ACK response for slave address and start data transfer
======================================================*/

            if ( 1 == TRC1 )
            {

/*----------------------------------------------
  data transmission mode operation
----------------------------------------------*/

                g_status1 = TX_MODE;        /* set data transmit mode       */
                if ( g_iica1_tx_cnt > 0 )
                {

/*----------------------------------------------
    there is the data to transmit
----------------------------------------------*/

                    IICA1 = *gp_iica1_tx_address; /* set transmit data            */
                    gp_iica1_tx_address++;        /* move pointer to next data    */
                    g_iica1_tx_cnt--;             /* count down data number       */
                }
                else
                {
/*----------------------------------------------
    no more data to transmit (transmission end)
----------------------------------------------*/

                    r_Config_IICA1_callback_master_sendend();
                }

            }   /* end of data transmission */
            else
            {

/*----------------------------------------------
    data receiving mode operation start
----------------------------------------------*/

                WTIM1 = 0;                  /* set 8bit wait mode           */
                ACKE1 = 1;
                
                IICA1 = DUMMY_DATA;         /* canceling wait and start Rx  */
                g_status1 = RX_MODE;        /* set receive mode flag        */

            }

        }   /* end of ACK for slave address */
        else
        {

/*----------------------------------------------
    NACK response for slave address
----------------------------------------------*/

            r_Config_IICA1_callback_master_error(MD_NACK);

        }   /* end of "no slave exist"  */

    }   /* end of slave address transmit operation  */
    else
    {

/*======================================================
    data transfer stage(STD1 is 0)
======================================================*/

        if ( 1 == TRC1 )                    /* check transmit or receive    */
        {

/*----------------------------------------------
    transmit operation mode
----------------------------------------------*/

            if ( 1 == ACKD1 )               /* check slave state            */
            {                               /* slave is ready for next data */

/*----------------------------------------------
  success data transfer and check remaining data
----------------------------------------------*/

                if ( g_iica1_tx_cnt > 0 )         /* check transmit data number   */
                {

/*----------------------------------------------
  prepare next transmit data
----------------------------------------------*/

                    IICA1 = *gp_iica1_tx_address; /* transmit data                */
                    gp_iica1_tx_address++;   /* move pointer to next data    */
                    g_iica1_tx_cnt--;        /* count down data number       */

                }   /* end of one data transmit */
                else
                {

/*----------------------------------------------
  data transmit complete, and execute callback function
----------------------------------------------*/
                    r_Config_IICA1_callback_master_sendend();

                }   /* complete transmission    */

            }   /* transmission end */
            else
            {
/*----------------------------------------------
    NACK error(NACK for data)
----------------------------------------------*/

                r_Config_IICA1_callback_master_error(MD_BUSY1);

            }
            
        }   /* end of transmit mode */
        else
        {

/*----------------------------------------------
    receive operation mode
----------------------------------------------*/

            if ( g_iica1_rx_cnt < g_iica1_rx_len )
            {

/*----------------------------------------------
store recieved data
----------------------------------------------*/

                *gp_iica1_rx_address = IICA1; /* store received data      */
                gp_iica1_rx_address++;        /* move store pointer       */
                g_iica1_rx_cnt++;             /* countup data number      */

                if ( g_iica1_rx_cnt == g_iica1_rx_len )
                {

/*----------------------------------------------
receive last data operation (send NACK and change to 9 clock wait)
----------------------------------------------*/
                    ACKE1 = 0;              /* NACK for last data       */
                    WTIM1 = 1;              /* change to 9clock wait    */
                    WREL1 = 1;              /* canceling wait to end    */
                }
                else
                {
                    WREL1 = 1;              /* start next reception     */
                }

            }   /* end of receive continue  */
            else
            {

/*----------------------------------------------
receive operation complete at 9th clock
----------------------------------------------*/

                r_Config_IICA1_callback_master_receiveend();    /* receive end callback */

            }   /* end of data number check of receive  */

        }   /* end of data receive  */

    }   /* master transfer operation end    */
}


/***********************************************************************************************************************
* Function Name: r_Config_IICA1_interrupt
* Description  : This function is INTIICA1 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void __near r_Config_IICA1_interrupt(void)
{
    if (0x80U == (IICS1 & _80_IICA_STATUS_MASTER))
    {
        r_Config_IICA1_master_handler();
    }
}

/* End user code. Do not edit comment generated here */
