/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_TAU0_3_user.c
* Component Version: 1.4.0
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_TAU0_3.
***********************************************************************************************************************/
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_TAU0_3.h"
/* Start user code for include. Do not edit comment generated here */
#include "Config_IICA0.h"
#include "Config_IICA1.h"
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code */
#ifndef USER_MODE
#pragma interrupt r_Config_TAU0_3_interrupt(vect=INTTM03, bank=RB1)
#endif
/* End user code */
/* Start user code for pragma. Do not edit comment generated here */
/* Start user code */
#ifdef USER_MODE
#pragma interrupt r_Config_TAU0_3_interrupt(vect=INTTM03, bank=RB1, enable=true)
#endif
/* End user code */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
#define SLADDR_HDC1080   ( 0b1000000 )  // I2C bus slave address of HDC1080
#define SLADDR_HS3001    ( 0b1000100 )  // I2C bus slave address of HS3001

volatile uint8_t    senser[4] =         // data buffer for sensor
{
    0x00,
    0x00,
    0x00,
    0x00
};

/*--------------------------------------
    result of HDC1080
----------------------------------------*/
volatile int16_t    temp1;              // Temperature data (0.1degree unit)
volatile uint8_t    humid1;             // Humidity data(unit %)

/*--------------------------------------
    result of HS3001
----------------------------------------*/
volatile int16_t    temp2;              // Temperature data (0.1degree unit)
volatile uint8_t    humid2;             // Humidity data(unit %)


volatile uint8_t    sensor_end;         // sensor complete flag

volatile uint8_t    m_time;             // sequence number
/*
    0x00 : sensor is standby
    0x01 : wait for Conversion Time(40milli seconds)
    0x02 : wait for data read(2milli seconds)
    0x03 : end of conversion(ready to read senser[])
    0xFF : status error
*/

extern volatile uint8_t sel_IICA;       // select IICA channel
extern volatile int8_t disp_line1[40];  // LCD display buffer aera line1
extern volatile int8_t disp_line2[40];  // LCD display buffer aera line2

/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_TAU0_3_Create_UserInit
* Description  : This function adds user code after initializing the TAU0 channel 3.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_TAU0_3_Create_UserInit(void)
{
    /* Start user code for user init. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_TAU0_3_interrupt
* Description  : This function is INTTM03 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
/* Start user code */
//----------------------------------------------------------------------
//
//  40milli seconds interval for sensor sequencer
//
//----------------------------------------------------------------------
/* End user code */
static void __near r_Config_TAU0_3_interrupt(void)
{
    /* Start user code for r_Config_TAU0_3_interrupt. Do not edit comment generated here */

    uint8_t     status;
    uint32_t    long_work;

/*======================================
    check timing
========================================*/
    switch (m_time)
    {

/*======================================
    HDC1080 is selected
========================================*/

        case 0x01:              // 40milli seconds from activate

/*--------------------------------------
    read data from HDC1080
----------------------------------------*/
            if ( 0 == sel_IICA )    // used IICA is 0
            {
                status = R_Config_IICA0_Master_Receive (
                    SLADDR_HDC1080*2,   // slave address of HDC1080
                    (uint8_t *)senser,  // address register value
                    0x04,               // read data is 4 bytes
                    0xFF);              // timeout is max value
            }
            else
            {                       // used IICA is 1
                status = R_Config_IICA1_Master_Receive (
                    SLADDR_HDC1080*2,   // slave address of HDC1080
                    (uint8_t *)senser,  // address register value
                    0x04,               // read data is 4 bytes
                    0xFF);              // timeout is max value
            }

            ++m_time;
            break;

        case 0x02:              // 10milli seconds from read
            if ( 0 == sel_IICA )// used IICA is 0
            {
                status = R_IICA0_poll();// check receive status
            }
            else
            {                   // used IICA is 1
                status = R_IICA1_poll();// check receive status
            }

            if ( NO_SLAVE == status )
            {
                --m_time;               // for retry to receive
                break;                  // not ready
            }

            if ( ON_COMMU == status )
            {
                break;                  // not complete
            }

/*--------------------------------------
    read data complete
----------------------------------------*/
            R_Config_TAU0_3_Stop();     // stop TM03

            if ( 0 == sel_IICA )// used IICA is 0
            {
                R_IICA0_StopCondition();// release IIC bus
            }
            else
            {                   // used IICA is 1
                R_IICA1_StopCondition();// release IIC bus
            }
/*--------------------------------------
   Calculating Temperature (0.1degreeC)
----------------------------------------*/
            long_work =  ( senser[0] * 0x100UL + senser[1] );
                // merge higher 8bits and lower 6bits
            long_work *= 1650UL;        // multiply 10 times ofMaximum temperature
            temp1 = (int)((long_work>> 16 ) - 400); // adjust offset(40degreeC)


/*--------------------------------------
   Calculating Humidity (%)
----------------------------------------*/
            long_work = ( senser[2] * 0x100UL + senser[3] );
                // merge higher 8bits and lower 6bits
            long_work *= 100UL;         // get percentage
            humid1 = (unsigned char)(long_work >> 16); // get humidity
            sensor_end = 0x01;          // set sensor complete flag
            m_time = 0x03;              // set complete data


/*======================================
    HS3001 is selected
========================================*/

        case 0x11:              // 40milli seconds from activate

/*--------------------------------------
    read data from HS3001
----------------------------------------*/
            if ( 0 == sel_IICA )    // used IICA is 0
            {
                status = R_Config_IICA0_Master_Receive (
                    SLADDR_HS3001*2,    // slave address of HS3001
                    (uint8_t *)senser,  // address register value
                    0x04,               // read data is 4 bytes
                    0xFF);              // timeout is max value
            }
            else
            {                       // used IICA is 1
                status = R_Config_IICA1_Master_Receive (
                    SLADDR_HS3001*2,    // slave address of HS3001
                    (uint8_t *)senser,  // address register value
                    0x04,               // read data is 4 bytes
                    0xFF);              // timeout is max value
            }

            ++m_time;
            break;

        case 0x12:              // 10milli seconds from read 
            if ( 0 == sel_IICA )    // used IICA is 0
            {
                status = R_IICA0_poll();// check receive status
            }
            else
            {                       // used IICA is 1
                status = R_IICA1_poll();// check receive status
            }

            if ( ON_COMMU == status )
            {
                break;                  // not complete
            }

/*--------------------------------------
    read data complete
----------------------------------------*/

            R_Config_TAU0_3_Stop();     // stop TM03

            if ( 0 == sel_IICA )    // used IICA is 0
            {
                R_IICA0_StopCondition();// release IIC bus
            }
            else
            {                       // used IICA is 1
                R_IICA1_StopCondition();// release IIC bus
            }

/*--------------------------------------
   Calculating Humidity (%)
----------------------------------------*/
            long_work = ( senser[0] * 0x100UL + senser[1] );
                                        // merge higher 6bits and lower 8bits
            long_work *= 100UL;         // get percentage
            humid2 = (unsigned char)(long_work >> 14); // get humidity

/*--------------------------------------
   Calculating Temperature ( 0.1degreeC )
----------------------------------------*/
            long_work =  ( senser[2] * 0x40UL + ( senser[3] >>2 ));
                                        // merge higher 8bits and lower 6bits
            long_work *= 1650;          // multiply 10 times of Maximum temperature
            temp2 = (int)((long_work >>14 ) -400); // adjust offset(40degreeC)
            sensor_end = 0x01;          // set sensor complete flag
            m_time = 0x03;              // set complete data+

        default:
            break;

    }   /* end of switch case   */

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/*******************************************************************************
* Function Name: R_sensor_start
* Description  : This function start sensor
* Arguments    : sensor
*               0x00 : HDC1080
*               0x01 : HS3001
* Return Value : status
*               0x00(SUCCESS)   : success to trigger sensor
*               0x8F(BUS_ERROR) : I2C bus is busy
*               0x80(NO_SLAVE)  : sennsor is not exist
********************************************************************************/
uint8_t R_sensor_start(uint8_t sensor_sel)
{

    uint8_t status;             // IICAn status

    sensor_end = 0x00;          // clear sensor complete flag
    senser[0] = 0x00;           // set address data for HDC1080/HS3001

/*=====================================================================
    trigger sensor to start measurement
=======================================================================*/

    if ( 0x00 == sensor_sel )
    {
/*---------------------------------------------------------------------
    trigger HDC1080(start to measure)
-----------------------------------------------------------------------*/
        if ( 0 == sel_IICA )    // used IICA is 0
        {
            status = R_Config_IICA0_Master_Send (
                SLADDR_HDC1080*2,   // slave address of HDC1080
                (uint8_t *)senser,  // address register value
                0x01,               // send data is 1 byte
                0xFF);              // timeout is max value
        }
        else
        {
            status = R_Config_IICA1_Master_Send (
                SLADDR_HDC1080*2,   // slave address of HDC1080
                (uint8_t *)senser,  // address register value
                0x01,               // send data is 1 byte
                0xFF);              // timeout is max value
        }

            m_time = 0x01;          // set active flag

    }
    else
    {
/*---------------------------------------------------------------------
    trigger HS3001(start to measure)
-----------------------------------------------------------------------*/
        if ( 0 == sel_IICA )    // used IICA is 0
        {
            status = R_Config_IICA0_Master_Send (
                SLADDR_HS3001*2,    // slave address of HS3001
                (uint8_t *)senser,  // address register value
                0x00,               // no send data
                0xFF);              // timeout is max value
        }
        else
        {                       // used IICA is 1
            status = R_Config_IICA1_Master_Send (
                SLADDR_HS3001*2,    // slave address of HS3001
                (uint8_t *)senser,  // address register value
                0x00,               // no send data
                0xFF);              // timeout is max value
        }

            m_time = 0x11;          // set active flag
    }

/*=====================================================================
    trigger sensor to start measurement
=======================================================================*/

    if ( MD_OK == status )          // success to send slave address
    {

/*---------------------------------------------------------------------
    wait for trigger end
-----------------------------------------------------------------------*/

        if ( 0 == sel_IICA )    // used IICA is 0
        {
            status = R_IICA0_wait_comend(0);// wait for end of command
        }
        else
        {                       // used IICA is 0
            status = R_IICA1_wait_comend(0);// wait for end of command
        }

        if ( SUCCESS == status )        // success to transmit
        {

/*---------------------------------------------------------------------
    start to wait for complete to measure 
-----------------------------------------------------------------------*/

            TDR03 = (40000 - 1);        // set 40milli seconds
            R_Config_TAU0_3_Start();    // start TM03
            while ( 0 == ( TE0L & 0x08 ) )
            {                           // wait for TM03 start
                NOP();
            }
            TDR03 = ( 10000 -1 );       // set 10 milli seconds interval

        }
        else
        {
            m_time = 0x00;              // clear active flag
            status = NO_SLAVE;
        }

    }
    else
    {
/*---------------------------------------------------------------------
    Bus error and fail to trigger measurement
-----------------------------------------------------------------------*/

        m_time = 0x00;                  // clear active flag
        status = BUS_ERROR;
    }
    
    return ( status );

}   /* end of function R_sensor_start   */

/* End user code. Do not edit comment generated here */
