/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_TAU0_4_user.c
* Component Version: 1.4.0
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_TAU0_4.
***********************************************************************************************************************/
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_TAU0_4.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code */
#ifndef USER_MODE
#pragma interrupt r_Config_TAU0_4_interrupt(vect=INTTM04, bank=RB1)
#endif
/* End user code */
/* Start user code for pragma. Do not edit comment generated here */
/* Start user code */
#ifdef USER_MODE
#pragma interrupt r_Config_TAU0_4_interrupt(vect=INTTM04, bank=RB1, enable=true)
#endif
/* End user code */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
volatile uint16_t   push_time;          // SW push time count
volatile uint8_t    second_f;           // one second flag
volatile uint8_t    minute_f;           // one minute flag
volatile uint8_t    s_push;             // short time push
volatile uint8_t    m_push;             // middle time push
volatile uint8_t    l_push;             // long time push
volatile uint8_t    sw_work;            // work for switch check
volatile uint8_t    sw_data;            // for SW input check
volatile uint8_t    time_cnt1;          // time control for 1 second
volatile uint8_t    time_cnt2;          // time coutrol for 1 minute
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_TAU0_4_Create_UserInit
* Description  : This function adds user code after initializing the TAU0 channel 4.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_TAU0_4_Create_UserInit(void)
{
    /* Start user code for user init. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_TAU0_4_interrupt
* Description  : This function is INTTM04 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
/* Start user code */
//----------------------------------------------------------------------
//
//  20milli seconds interval for SW check
//
//----------------------------------------------------------------------
/* End user code */
static void __near r_Config_TAU0_4_interrupt(void)
{
    /* Start user code for r_Config_TAU0_4_interrupt. Do not edit comment generated here */

    sw_work <<= 1;                  // shift old switch data left
    if ( 0 != USER_SW )             // check SW status
    {
        sw_work++;                  // set new bit to LSB of sw_work
    }

    sw_data = (sw_work & 0x03);     // extract check timing data

//----------------------------------------------------------------------
//  1 second time detect
//----------------------------------------------------------------------

    if ( 50 <= ++time_cnt1 )
    {
        second_f = 1;               // set 1 second flag
        time_cnt1 = 0x00;           // clear counter

        if ( 60 <= ++time_cnt2 )
        {
            minute_f = 1;           // set 1 minute flag
            time_cnt2 = 0x00;       // clear counter
        }

    }

/*----------------------------------------------------------------------
    switch status handling
-----------------------------------------------------------------------*/

    switch ( sw_data )
    {
        case 0b10:                  // switch is just pushed
            push_time = 0x0000;     // clear push time
            break;

        case 0b00:                  // keep pushing
            ++push_time;            // count up push time
            break;

        case 0b01:                  // release key
            if ( 20 > push_time )   // less than 400 millis
            {
                ++s_push;           // set short time push flag
            }
            else if ( 100 > push_time ) // less than 2 second
            {
                ++m_push;           // set middle time push flag
            }
            else
            {
                ++l_push;           // set long time push flag 
            }
            break;
    }
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
