/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2021, 2023 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_TAU0_6_user.c
* Component Version: 1.4.0
* Device(s)        : R7F100GLGxFB
* Description      : This file implements device driver for Config_TAU0_6.
***********************************************************************************************************************/
/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_TAU0_6.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code */
#ifndef USER_MODE
#pragma interrupt r_Config_TAU0_6_interrupt(vect=INTTM06, bank=RB2)
#endif
/* End user code */
/* Start user code for pragma. Do not edit comment generated here */
/* Start user code */
#ifdef USER_MODE
#pragma interrupt r_Config_TAU0_6_interrupt(vect=INTTM06, bank=RB2, enable=true)
#endif
/* End user code */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
volatile uint32_t g_millisecond;        /* delay time counter                   */

/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_TAU0_6_Create_UserInit
* Description  : This function adds user code after initializing the TAU0 channel 6.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_TAU0_6_Create_UserInit(void)
{
    /* Start user code for user init. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_Config_TAU0_6_interrupt
* Description  : This function is INTTM06 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void __near r_Config_TAU0_6_interrupt(void)
{
    /* Start user code for r_Config_TAU0_6_interrupt. Do not edit comment generated here */

    ++g_millisecond;            // count up millisecond from start

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */

/*******************************************************************************
* Function Name: millis
* Description  : This function return elapsed time(milli seconds)
* Arguments    : None
* Return Value : elapsed time( milli seconds )
********************************************************************************/
uint32_t millis(void)
{

    uint32_t work;

/*  for exclusive control               */
    DI();                               /* prohibit interrupt           */
    work = g_millisecond;               /* read elapsed time(milli)     */
    EI();                               /* enable interrupt             */
/*  end of exclusive control            */

    return(work);                       /* set elapsed time(milli)      */

}


/*******************************************************************************
* Function Name: micros
* Description  : This function return elapsed time(micro seconds)
* Arguments    : None
* Return Value : 16bits elapsed time( micro seconds )
********************************************************************************/
uint16_t micros(void)
{

    uint16_t time1;                     /* 1st read data                */
    uint16_t time2;                     /* milli second data            */

/*  for exclusive control               */
    DI();                               /* prohibit interrupt           */
    time1  = 1000 - TCR06;              /* get micro second data        */
    time2  = (uint16_t)(0xFFFF & g_millisecond ); /* get milli second   */

    if ( ( 1 == TMIF06 )&&( time1 < 2 ) ) /* check if just countup      */
    {                                   /* when coutup add 1000micro    */
        time1 += 1000;                  /* adjust micro second          */
    } 

    EI();                               /* enable interrupt             */
/*  end of exclusive control            */

    time2  = (uint16_t)(time2 * 1000);  /* make micro seconds data      */
    time2  = (uint16_t)( time2 + time1 );  /* add low digit data        */

    return(time2);                      /* return with elapsed time     */

}

/* End user code. Do not edit comment generated here */
